module PackagecloudDebtools
  class Digester
    class Digest
      def initialize
        @md5sum    = ::Digest::MD5.new
        @sha1sum   = ::Digest::SHA1.new
        @sha256sum = ::Digest::SHA256.new
        @sha512sum = ::Digest::SHA512.new
      end

      def update(with)
        @md5sum.update(with)
        @sha1sum.update(with)
        @sha256sum.update(with)
        @sha512sum.update(with)
      end

      def md5
        @md5sum.hexdigest
      end

      def sha1
        @sha1sum.hexdigest
      end

      def sha256
        @sha256sum.hexdigest
      end

      def sha512
        @sha512sum.hexdigest
      end
    end

    class << self
      def from_file(path)
        Digest.new.tap do |d|
          ::File.open(path, 'rb') do |io|
            buf = ""
            while io.read(4096, buf)
              d.update(buf)
            end
          end
        end
      end

      def from_io(io)
        Digest.new.tap do |d|
          buf = ""
          while io.read(4096, buf)
            d.update(buf)
          end
        end
      end

      def from_archive(archive)
        Digest.new.tap do |d|
          archive.read_data(4096) do |data|
            d.update(data)
          end
        end
      end
    end
  end
end
